<?php
/**************************************************************************
 * $Id: FormTools.lib.php 4278 2010-08-26 08:55:11Z renner $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


class InputConstraint {
  public function check($val, &$res)
  {
    return false;
  }
}

class InputIntConstraint extends InputConstraint {
  private $min = null;
  private $max = null;


  public function InputIntConstraint($min, $max)
  {
    /* setup validation */
    if (! is_null($min) && ! is_int($min)) die "min is not an int";
    if (! is_null($max) && ! is_int($min)) die "max is not an int";
  
    $this->min = $min;
    $this->max = $max;
  }


  public function check($val, &$res)
  {
    // check type
    if (! is_int($val)) {
      addFormError($res, sprintf('Eingabe ist keine Zahl'));
      return false;
    }
    
    // check min/max
    if (! is_null($this->min) && $val < $this->min) {
      addFormError($res, sprintf('Wert ist zu klein (Minimum: %d)', $this->min));
      return false;
    }
    if (! is_null($this->max) && ! $val > $this->max) {
      addFormError($res, sprintf('Wert ist zu groß (Minimum: %d)', $this->max));
      return false;
    }
    
    return true;
  }
}

class InputStringConstraint extends InputConstraint {
  private $min     = null;
  private $max     = null;
  private $pattern = null;
  
  
  public function InputStringConstraint($min, $max, $pat)
  {
    /* setup validation */
    if (! is_null($min) && ! is_int($min)) die "min is not an int";
    if (! is_null($max) && ! is_int($min)) die "max is not an int";
    if (! is_null($pat) && ! is_string($pat)) die "pattern is not a string";
  
    $this->min     = $min;
    $this->max     = $max;
    $this->pattern = $pat;
  }
  
  
  public function check($val, &$res)
  {
    // check type
    if (! is_string($val)) {
      addFormError($res, sprintf('Eingabe ist kein Text'));
      return false;
    }
    
    // check length
    if (! is_null($this->min) && strlen($val) < $this->min) {
      addFormError($res, sprintf('Eingabe muss mindestens %d Zeichen lang sein', $this->min));
      return false;
    }
    if (! is_null($this->max) && ! strlen($val) > $this->max) {
      addFormError($res, sprintf('Eingabe darf höchstens %d Zeichen lang sein', $this->max));
      return false;
    }
    
    // check pattern
    if (! is_null($this->pattern) && ! preg_match($this->pattern, $val)) {
      addFormError($res, sprintf('Eingabe erfüllt nicht das vorgebene Muster'));
      return false;
    }
    
    return true;
  }
}

class InputDateConstraint extends InputConstraint {
  public function InputDateConstraint()
  {
    /* void */
  }


  public function check($val, &$res)
  {
    $parts = split('.', $val);
    
    // check day, year, month exist
    if (count($parts) != 3) {
      addFormError($res, sprintf("Datum besitzt nicht das Format Tag.Monat.Jahr"));
      return false;
    }

    // check, if date is valid at all
    if (! checkdate($parts[1], $parts[0], $parts[2])) {
      addFormError($res, sprintf("Datum ist ungültig"));
      return false;
    }
    
    // check, if date is not in the past
    // date in past no error, just a warning
    $inp = $parts[2].$parts[1].$parts[0];
    $now = date("Ymd");
    if ($now > $inp) {
      addWarning($res, sprintf("Eingegebenes Datum liegt in der Vergangenheit"));
    }
    
    return true;
  }
}


class InputSetConstraint extends InputConstraint {
  private $set = null;
  

  public function InputSetConstraint($set)
  {
    if (! is_array($set) || count($set <= 0)) die "set is no set or empty";
    
    $this->set = $set;
  }


  public function check($val, &$res)
  {
    // check type
    if (! is_string($val) && ! is_int($val)) {
      addFormError($res, sprintf('Eingabe ungültig'));
      return false;
    }

    // check, if val is in set
    if (! in_array($val, $this->set)) {
      addFormError($res, sprintf('Wert nicht erlaubt'));
      return false;
    }
    
    return true;
  }
}

/*
function validateInput($val, &$constr, &$res)
{
  // TODO preprocess $value
  // - strip
  // - gpc
  // - ....

  return $constr->check($val, $res);
}
*/

?>

